package test.uses.multi;

import javax.security.auth.callback.*;
import javax.sql.*;

public class UsesMulti {
	
	public static Callback foo(){
		return null;
	}
	
	public static DataSource dataSource() {
		return null;
	}

}
