package aQute.bnd.main;

import java.io.*;
import java.util.*;

public class Interactive {

	public Interactive(PrintStream out, InputStream in, Map<String,String> parameters) {
		
	}

	public boolean go() {
		// TODO Auto-generated method stub
		return false;
	}

}
