package test.activator;

import org.osgi.framework.*;

class DefaultVisibilityActivator implements BundleActivator {

	@Override
	public void start(BundleContext context) throws Exception { }

	@Override
	public void stop(BundleContext context) throws Exception { }

}
