package test.activator;

import org.osgi.framework.*;

public class NotAnActivator {

	public void start(BundleContext context) throws Exception { }

	public void stop(BundleContext context) throws Exception { }

}
