package aQute.bnd.annotation.licenses;

import aQute.bnd.annotation.headers.*;

/**
 * An annotation to indicate that the type depends on the Simplified BSD or
 * FreeBSD License. Applying this annotation will add a Bundle-License clause.
 */
@BundleLicense(name = "http://opensource.org/licenses/BSD-2-Clause", link = "http://en.wikipedia.org/wiki/BSD_licenses", description = "Simplified BSD or FreeBSD License")
public @interface BSD_2_Clause {}
