package test.uses.annotation;

import javax.security.auth.Subject;

import test.uses.annotation.annotation.Annotation;

@Annotation
public class UsesAnnotation {
	
	private UsesAnnotation(Subject subject) {
		
	}

}
